@@ -1576,7 +1576,7 @@ GSList *sim_parse_app_template_entries(const unsigned char *buffer, int len)
 
 		memcpy(app.aid, aid, app.aid_len);
 
-		app.type = (app.aid[5] << 8) & app.aid[6];
+		app.type = (app.aid[5] << 8) | app.aid[6];
 
 		/* Find the label (optional) */
 		label = ber_tlv_find_by_tag(dataobj, 0x50, dataobj_len,
